#!/usr/bin/env python
##################################################
# Gnuradio Python Flow Graph
# Title: Top Block
# Generated: Thu Aug 23 15:24:27 2012
##################################################

from gnuradio import audio
from gnuradio import blks2
from gnuradio import eng_notation
from gnuradio import gr
from gnuradio import window
from gnuradio.eng_option import eng_option
from gnuradio.gr import firdes
from gnuradio.wxgui import fftsink2
from gnuradio.wxgui import forms
from grc_gnuradio import wxgui as grc_wxgui
from optparse import OptionParser
import acars
import osmosdr
import wx

class top_block(grc_wxgui.top_block_gui):

	def __init__(self):
		grc_wxgui.top_block_gui.__init__(self, title="Top Block")

		##################################################
		# Variables
		##################################################
		self.samp_rate = samp_rate = 48000
		self.rf_freq = rf_freq = 131725000
		self.ch0ifgain = ch0ifgain = 20
		self.ch0gain = ch0gain = 30
		self.audiogain = audiogain = 150

		##################################################
		# Blocks
		##################################################
		_ch0gain_sizer = wx.BoxSizer(wx.VERTICAL)
		self._ch0gain_text_box = forms.text_box(
			parent=self.GetWin(),
			sizer=_ch0gain_sizer,
			value=self.ch0gain,
			callback=self.set_ch0gain,
			label='ch0gain',
			converter=forms.float_converter(),
			proportion=0,
		)
		self._ch0gain_slider = forms.slider(
			parent=self.GetWin(),
			sizer=_ch0gain_sizer,
			value=self.ch0gain,
			callback=self.set_ch0gain,
			minimum=0,
			maximum=50,
			num_steps=51,
			style=wx.SL_HORIZONTAL,
			cast=float,
			proportion=1,
		)
		self.Add(_ch0gain_sizer)
		_audiogain_sizer = wx.BoxSizer(wx.VERTICAL)
		self._audiogain_text_box = forms.text_box(
			parent=self.GetWin(),
			sizer=_audiogain_sizer,
			value=self.audiogain,
			callback=self.set_audiogain,
			label='audiogain',
			converter=forms.float_converter(),
			proportion=0,
		)
		self._audiogain_slider = forms.slider(
			parent=self.GetWin(),
			sizer=_audiogain_sizer,
			value=self.audiogain,
			callback=self.set_audiogain,
			minimum=0,
			maximum=1000,
			num_steps=100,
			style=wx.SL_HORIZONTAL,
			cast=float,
			proportion=1,
		)
		self.Add(_audiogain_sizer)
		self.wxgui_fftsink2_0 = fftsink2.fft_sink_c(
			self.GetWin(),
			baseband_freq=rf_freq,
			y_per_div=10,
			y_divs=10,
			ref_level=0,
			ref_scale=2.0,
			sample_rate=samp_rate*24,
			fft_size=1024,
			fft_rate=3,
			average=False,
			avg_alpha=None,
			title="FFT Plot",
			peak_hold=False,
		)
		self.Add(self.wxgui_fftsink2_0.win)
		self.osmosdr_source_c_0 = osmosdr.source_c( args="nchan=" + str(1) + " " + ""  )
		self.osmosdr_source_c_0.set_sample_rate(samp_rate*24)
		self.osmosdr_source_c_0.set_center_freq(rf_freq, 0)
		self.osmosdr_source_c_0.set_freq_corr(0, 0)
		self.osmosdr_source_c_0.set_gain_mode(1, 0)
		self.osmosdr_source_c_0.set_gain(ch0gain, 0)
		self.low_pass_filter_0 = gr.fir_filter_ccf(6, firdes.low_pass(
			2, samp_rate*24, 500000, 150000, firdes.WIN_HAMMING, 6.76))
		self.gr_multiply_const_vxx_0 = gr.multiply_const_vff((audiogain, ))
		self.gr_file_sink_0 = gr.file_sink(gr.sizeof_float*1, "/home/jmfriedt/test2.bin")
		self.gr_file_sink_0.set_unbuffered(False)
		_ch0ifgain_sizer = wx.BoxSizer(wx.VERTICAL)
		self._ch0ifgain_text_box = forms.text_box(
			parent=self.GetWin(),
			sizer=_ch0ifgain_sizer,
			value=self.ch0ifgain,
			callback=self.set_ch0ifgain,
			label="ch0ifgain",
			converter=forms.float_converter(),
			proportion=0,
		)
		self._ch0ifgain_slider = forms.slider(
			parent=self.GetWin(),
			sizer=_ch0ifgain_sizer,
			value=self.ch0ifgain,
			callback=self.set_ch0ifgain,
			minimum=0,
			maximum=50,
			num_steps=51,
			style=wx.SL_HORIZONTAL,
			cast=float,
			proportion=1,
		)
		self.Add(_ch0ifgain_sizer)
		self.blks2_am_demod_cf_0 = blks2.am_demod_cf(
			channel_rate=samp_rate*4,
			audio_decim=4,
			audio_pass=5000,
			audio_stop=5500,
		)
		self.audio_sink_0 = audio.sink(samp_rate, "", True)
		self.acars_decodeur_0 = acars.decodeur(150,"/tmp/log_jmf.txt")

		##################################################
		# Connections
		##################################################
		self.connect((self.osmosdr_source_c_0, 0), (self.wxgui_fftsink2_0, 0))
		self.connect((self.blks2_am_demod_cf_0, 0), (self.audio_sink_0, 0))
		self.connect((self.low_pass_filter_0, 0), (self.blks2_am_demod_cf_0, 0))
		self.connect((self.osmosdr_source_c_0, 0), (self.low_pass_filter_0, 0))
		self.connect((self.blks2_am_demod_cf_0, 0), (self.gr_multiply_const_vxx_0, 0))
		self.connect((self.blks2_am_demod_cf_0, 0), (self.gr_file_sink_0, 0))
		self.connect((self.gr_multiply_const_vxx_0, 0), (self.acars_decodeur_0, 0))

	def get_samp_rate(self):
		return self.samp_rate

	def set_samp_rate(self, samp_rate):
		self.samp_rate = samp_rate
		self.osmosdr_source_c_0.set_sample_rate(self.samp_rate*24)
		self.low_pass_filter_0.set_taps(firdes.low_pass(2, self.samp_rate*24, 500000, 150000, firdes.WIN_HAMMING, 6.76))
		self.wxgui_fftsink2_0.set_sample_rate(self.samp_rate*24)

	def get_rf_freq(self):
		return self.rf_freq

	def set_rf_freq(self, rf_freq):
		self.rf_freq = rf_freq
		self.osmosdr_source_c_0.set_center_freq(self.rf_freq, 0)
		self.wxgui_fftsink2_0.set_baseband_freq(self.rf_freq)

	def get_ch0ifgain(self):
		return self.ch0ifgain

	def set_ch0ifgain(self, ch0ifgain):
		self.ch0ifgain = ch0ifgain
		self._ch0ifgain_slider.set_value(self.ch0ifgain)
		self._ch0ifgain_text_box.set_value(self.ch0ifgain)

	def get_ch0gain(self):
		return self.ch0gain

	def set_ch0gain(self, ch0gain):
		self.ch0gain = ch0gain
		self._ch0gain_slider.set_value(self.ch0gain)
		self._ch0gain_text_box.set_value(self.ch0gain)
		self.osmosdr_source_c_0.set_gain(self.ch0gain, 0)

	def get_audiogain(self):
		return self.audiogain

	def set_audiogain(self, audiogain):
		self.audiogain = audiogain
		self._audiogain_slider.set_value(self.audiogain)
		self._audiogain_text_box.set_value(self.audiogain)
		self.gr_multiply_const_vxx_0.set_k((self.audiogain, ))

if __name__ == '__main__':
	parser = OptionParser(option_class=eng_option, usage="%prog: [options]")
	(options, args) = parser.parse_args()
	tb = top_block()
	tb.Run(True)

